@@ -16,6 +16,7 @@ from .core import \
     StringPattern, \
     Value, \
     _
+from .error import MatchError
 from .match import match
 from .overload import case_distinction, Match
 from .patterns import \
@@ -36,8 +37,7 @@ from .patterns import \
     Object, \
     Regex, \
     Returns, \
-    Transformed, \
-    Truish
+    Transformed
 from .try_match import Case, Default
 
 __all__ = [
@@ -62,6 +62,7 @@ __all__ = [
     'match',
     'case_distinction',
     'Match',
+    'MatchError',
     'Case',
     'Default',
 
@@ -83,5 +84,4 @@ __all__ = [
     'Regex',
     'Returns',
     'Transformed',
-    'Truish',
 ]
