@@ -5,11 +5,12 @@ from .match import match
 
 
 class CaseExpr:
-    def __init__(self, value):
+    def __init__(self, value, **kwargs):
         self._value = value
+        self._kwargs = kwargs
 
-    def of(self, pattern, then) -> CaseExpr:
-        result = match(self._value, pattern)
+    def of(self, pattern, then, **kwargs) -> CaseExpr:
+        result = match(self._value, pattern, **{**self._kwargs, **kwargs})
         if result:
             if callable(then):
                 return CaseExprEnd(invoke(then, result))
@@ -27,7 +28,7 @@ class CaseExpr:
 
 class CaseExprEnd(CaseExpr):
 
-    def of(self, pattern, then) -> CaseExpr:
+    def of(self, pattern, then, **kwargs) -> CaseExpr:
         return self
 
     def otherwise(self, then):
