@@ -4,11 +4,11 @@ import re
 from typing import Callable, Optional, Dict, Any
 
 from ._util import get_arg_types, get_return_type, get_kwarg_types
-from .core import Pattern, MatchContext, MatchResult, StringPattern, OneOf
+from .core import Pattern, MatchContext, MatchResult, StringPattern, OneOf, Nested, Underscore
 
 
 class Check(Pattern):
-    def __init__(self, condition, /):
+    def __init__(self, condition):
         self._condition = condition
 
     def match(self, value, *, ctx: MatchContext, strict: bool) -> MatchResult:
@@ -16,20 +16,36 @@ class Check(Pattern):
 
 
 class Regex(Pattern, StringPattern):
-    def __init__(self, regex, /):
+    def __init__(self, regex, *, bind_groups: bool = True, capture_wildcards: bool = False):
         self._regex: re.Pattern = re.compile(regex)
+        self._bind_groups = bind_groups
+        if capture_wildcards:
+            self._wildcards = tuple(Underscore() for _i in range(0, self._regex.groups))
+        else:
+            self._wildcards = tuple([])
 
     def match(self, value, *, ctx: MatchContext, strict: bool) -> MatchResult:
-        return ctx.match_if(bool(self._regex.fullmatch(value)))
+        try:
+            result = self._regex.fullmatch(value)
+        except TypeError:
+            return ctx.no_match()
+        if not result:
+            return ctx.no_match()
+        if self._bind_groups:
+            for k, v in result.groupdict().items():
+                ctx[k] = v
+        for ix, wc in enumerate(self._wildcards):
+            ctx.record(wc, result.group(ix + 1))
+        return ctx.matches()
 
     def string_match(self, remaining, *, ctx: MatchContext) -> Optional[str]:
-        if result := self._regex.match(remaining):
-            return result.group(0)
-        return None
-
-    @property
-    def regex(self) -> re.Pattern:
-        return self._regex
+        result = self._regex.match(remaining)
+        if not result:
+            return None
+        if self._bind_groups:
+            for k, v in result.groupdict():
+                ctx[k] = v
+        return result.group(0)
 
 
 class InstanceOf(Pattern):
@@ -60,7 +76,7 @@ class Between(Pattern):
 
 
 class Length(Pattern):
-    def __init__(self, length=None, /, at_least: int = None, at_most: int = None):
+    def __init__(self, length=None, at_least: int = None, at_most: int = None):
         if length is not None:
             if at_least is not None or at_most is not None:
                 raise ValueError("If length is given, 'at_least' or 'at_most' must not be given.")
@@ -84,8 +100,8 @@ class Contains(Pattern):
         return ctx.match_if(self._needle in value)
 
 
-class Transformed(Pattern):
-    def __init__(self, f: Callable, pattern, /):
+class Transformed(Pattern, Nested):
+    def __init__(self, f: Callable, pattern):
         self._f = f
         self._pattern = pattern
 
@@ -97,57 +113,80 @@ class Transformed(Pattern):
             return ctx.no_match()
         return ctx.match(transformed, self._pattern)
 
+    def descend(self, f):
+        return Transformed(f=self._f, pattern=f(self._pattern))
 
-class Arguments(Pattern):
+
+class Arguments(Pattern, Nested):
     def __init__(self, *arg_patterns, **kwargs):
         self._pattern = list(arg_patterns)
         self._kwargs: Dict[str, Any] = kwargs
 
     def match(self, value, *, ctx: MatchContext, strict: bool) -> MatchResult:
         if self._pattern:
-            if not (result := ctx.match(get_arg_types(value), self._pattern)):
+            result = ctx.match(get_arg_types(value), self._pattern)
+            if not result:
                 return result
         return ctx.match(get_kwarg_types(value), self._kwargs, strict)
 
+    def descend(self, f):
+        t_pattern = list(f(p) for p in self._pattern)
+        t_kwargs = {}
+        for k, v in self._kwargs.items():
+            t_kwargs[k] = f(v)
+        return Arguments(*t_pattern, **t_kwargs)
 
-class Returns(Pattern):
-    def __init__(self, pattern, /):
+
+class Returns(Pattern, Nested):
+    def __init__(self, pattern):
         self._pattern = pattern
 
     def match(self, value, *, ctx: MatchContext, strict: bool) -> MatchResult:
         return ctx.match(get_return_type(value), self._pattern)
 
+    def descend(self, f):
+        return Returns(pattern=f(self._pattern))
 
-class Each(Pattern):
-    def __init__(self, pattern, /, *, at_least: int = 0):
+
+class Each(Pattern, Nested):
+    def __init__(self, pattern, *, at_least: int = 0):
         self._pattern = pattern
         self._at_least = at_least
 
     def match(self, value, *, ctx: MatchContext, strict: bool) -> MatchResult:
         count = 0
         for item in value:
-            if not (result := ctx.match(item, self._pattern)):
+            result = ctx.match(item, self._pattern)
+            if not result:
                 return result
             count += 1
         return ctx.match_if(count >= self._at_least)
 
+    def descend(self, f):
+        return Each(pattern=f(self._pattern), at_least=self._at_least)
 
-class EachItem(Pattern):
-    def __init__(self, key_pattern, value_pattern, /):
+
+class EachItem(Pattern, Nested):
+    def __init__(self, key_pattern, value_pattern):
         self._key_pattern = key_pattern
         self._value_pattern = value_pattern
 
     def match(self, value, *, ctx: MatchContext, strict: bool) -> MatchResult:
         for k, v in value.items():
-            if not (result := ctx.match(k, self._key_pattern)):
+            result = ctx.match(k, self._key_pattern)
+            if not result:
                 return result
-            if not (result := ctx.match(v, self._value_pattern)):
+            result = ctx.match(v, self._value_pattern)
+            if not result:
                 return result
         return ctx.matches()
 
+    def descend(self, f):
+        return EachItem(key_pattern=f(self._key_pattern), value_pattern=f(self._value_pattern))
 
-class At(Pattern):
-    def __init__(self, path, pattern, /):
+
+class At(Pattern, Nested):
+    def __init__(self, path, pattern):
         if isinstance(path, str):
             self._path = path.split(".")
         else:
@@ -162,22 +201,32 @@ class At(Pattern):
                 return ctx.no_match()
         return ctx.match(value, self._pattern)
 
+    def descend(self, f):
+        return At(path=self._path, pattern=f(self._pattern))
 
-class Object(Pattern):
+
+class Object(Pattern, Nested):
     def __init__(self, **kwargs):
         self._items = kwargs
 
     def match(self, value, *, ctx: MatchContext, strict: bool) -> MatchResult:
         return ctx.match(value, self._items, strict=strict)
 
+    def descend(self, f):
+        items = {}
+        for k, v in self._items.items():
+            items[k] = f(v)
+        return Object(**items)
 
-class Truish(Pattern):
-    """Deprecated, use IsTruish"""
 
+class _IsTruish(Pattern):
     def match(self, value, *, ctx: MatchContext, strict: bool) -> MatchResult:
         return ctx.match_if(value)
 
 
+IsTruish = _IsTruish()
+
+
 # noinspection PyPep8Naming
 def NoneOf(*args) -> Pattern:
     return ~OneOf(args)
@@ -188,6 +237,5 @@ def Maybe(pattern) -> Pattern:
     return OneOf(pattern, ...)
 
 
-IsTruish = Truish
 IsNumber = (InstanceOf(int) & ~InstanceOf(bool)) | InstanceOf(float) | InstanceOf(decimal.Decimal)
 IsString = InstanceOf(str)
